package com.jwipc.sys.convert;

import com.jwipc.config.MapperStructConfig;
import com.jwipc.sys.dto.*;
import com.jwipc.sys.entity.User;
import org.mapstruct.Mapper;

import java.util.List;


@Mapper(componentModel = "spring", uses = MapperStructConfig.class)
public interface UserConvert {
    List<UserPageListDTO> entities2PageListDTO(List<User> users);


    User addDto2Entity(UserAddDTO userAddDTO);

    UserEditPageDTO entity2EditPageDTO(User user);

    User editDTO2entity(UserEditDTO userEditDTO);

    User createUserDto2Entities(CreateUserDTO createUserDTO);

}
